#!/usr/bin/env python2
from __future__ import with_statement
from scipy import *
import numpy as np
import pylab
import sys, os, re, glob
import load, getChi
from load import getFreqs
import scipy.stats
from numpy.random import choice as randChoice
from numpy.random import shuffle
from essentiality import seqLoad, getEnergy, hamming, loadSites, loadEssentialityData
import matplotlib as mpl

mpl.rc('text', usetex=False)
mpl.rc('font', **{'family':'sans-serif', 'sans-serif': 'Arial', 'serif': 'Times New Roman'})

cc = scipy.stats.spearmanr
sem = lambda x: scipy.stats.sem(x) if len(x) > 1 else nan
mean = lambda x: np.mean(x) if len(x) > 0 else nan
var = lambda x: np.var(x) if len(x) > 0 else nan


outdir = 'results'

def dE(str1, str2, em):
    return getEnergy(em, str1) - getEnergy(em, str2)

def loadEntropy(sitedir, essential, tested):

    esstfs = []
    unesstfs = []
    untesttfs = []
    nBases = 4
    tffiles = glob.glob('{}/*'.format(sitedir))
    print "Entropy Files:", len(tffiles)
    for tffile in tffiles:
        tf = os.path.basename(tffile)
        try:
            regulatedsites = loadSites(tffile)
        except:
            print "skipping entropy " + tf
            continue
        sites = array([seqLoad(seqs['recomputed']) for gene, pos, dist, seqs in regulatedsites if 'recomputed' in seqs])
        f = getFreqs(sites, nBases)
        
        nSeqs = sites.shape[0]
        
        if nSeqs < 10:
            continue

        ecorr = (nBases-1)/(2*log(2)*nSeqs)
        f = f/sum(f,1)[:,newaxis]
        old_settings = seterr(divide='ignore', invalid='ignore')
        R = log2(nBases) - (-sum(nan_to_num(f*log2(f)),1)+ecorr)
        seterr(**old_settings)
        totR = sum(R)/sites.shape[1]

        if totR < 0:
            print tf, R, nSeqs

        if tf in essential:
            esstfs.append(totR)
            #print 'Essential', tf, totR
        elif tf in tested:
            unesstfs.append(totR)
            #print 'Nonessential', tf, totR
        else:
            untesttfs.append(totR)
            #print 'Unknown', tf, totR
    return esstfs, unesstfs, untesttfs

def plots(data, varname):
    startx, endx = np.min(data.flatten())-1, np.max(data.flatten())+1

    pylab.plot(data[:,1], data[:,2] , '.')
    pylab.plot([startx,endx], [startx,endx], 'k-')
    pylab.xlim(startx, endx)
    pylab.ylim(startx, endx)
    pylab.xlabel(r'$%s_{\mathdefault{essential}}$' % varname)
    pylab.ylabel(r'$%s_{\mathdefault{non-essential}}$' % varname)

def tonum(x):
    try:
        return float(x)
    except:
        return nan

def nullHypothesis(d1, d2, outf=sys.stdout, prop=mean):
    data = r_[d1, d2]
   
    m1 = prop(d1) if len(d1) != 0 else nan
    m2 = prop(d2) if len(d2) != 0 else nan
    print >>outf, 'Ess: {: .3f}  Non: {: .3f}'.format( m1, m2)
    if isnan(m1) or isnan(m2):
        print >>outf, 'P(>=):', nan
        return nan

    ld1 = len(d1)
    def sample():
        shuffle(data)
        return prop(data[:ld1]) - prop(data[ld1:])
    nulldiff = array([sample() for i in range(int(1e6))])

    p = sum((abs(nulldiff) - abs(m1-m2)) >= -1e-3*abs(nulldiff))/float(len(nulldiff)) #1e-3 accounts for floating point error
    print >>outf, 'P(>=):', p
    return p

def main():
    pylab.rcParams['figure.subplot.bottom'] = 0.15

    essential, tested, rates, expr, cons = loadEssentialityData()
    
    esstfs = []
    unesstfs = []
    untesttfs = []
    nBases = 4
    meane = []
    vare = []
    meanexpr = []
    meangrowth = []
    meankaks = []
    meanham = []
    meandivE = []
    sqrtmeandivE = []
    tfs = []
    nseq = []
    epvals = []
    hpvals = []
    depvals = []
    for emfile in glob.glob(os.path.join(sys.argv[1], '*')):
    #for emfile in [os.path.join(sys.argv[1], 'AFT1')]:
        em = loadtxt(emfile)
        tf = os.path.basename(emfile)
        tffile = os.path.join(sys.argv[2], tf)
        if not os.path.exists(tffile):
            print "skipping" + tffile
            continue
        tfs.append(tf)
        print >>sys.stderr, "Essentiality analysis for {}".format(tf)
        print tf

        #load gene to  TFBS map
        regsites = loadSites(tffile)
        
        energies = dict([(gene, getEnergy(em, sites['recomputed'].strip())) for gene, pos, dist, sites in regsites])
        ham = dict([(gene, hamming(sites['cerevisiae'], sites['paradoxus'])) for gene, pos, dist, sites in regsites if 'paradoxus' in sites])
        divE = dict([(gene, dE(sites['cerevisiae'], sites['paradoxus'], em)**2) for gene, pos, dist, sites in regsites if 'paradoxus' in sites])
        alle = [e for gene, e in energies.iteritems()]

        regulated = energies.keys()

        if mean(alle) > -5: 
            print "Skipping", tf
            continue
        unknown = [gene for gene, e in energies.iteritems() if gene not in tested]
        
        try:
            outf = open(os.path.join(outdir, tf, 'essnoness.txt'), 'wt')
        except:
            print "Skipping", tf
            continue

        essentiale = [e for gene, e in energies.iteritems() if gene in essential]
        nonessentiale = [e for gene, e in energies.iteritems() if gene not in essential and gene in tested ]
        meane.append((mean(essentiale), mean(nonessentiale), sem(essentiale), sem(nonessentiale)))
        vare.append((var(essentiale), var(nonessentiale)))
        nseq.append(max(len(essentiale), len(nonessentiale)))

        essentialx = [expr[gene] for gene in regulated if gene in essential and gene in expr]
        nonessentialx = [expr[gene] for gene in regulated if gene not in essential and gene in tested and gene in expr]
        meanexpr.append((mean(essentialx), mean(nonessentialx), sem(essentialx), sem(nonessentialx)))
        print >>outf, "Number of genes in dataset:"
        print >>outf, "Expr: {nreggrowth} ({nrress} ess, {nrrnon} non)".format(
                      nreg=len(energies.keys()), 
                      nreggrowth=sum([k in expr for k in energies.keys()]), 
                      nrress=sum([(k in essential and k in expr) for k in energies.keys()]), 
                      nrrnon=sum([((k not in essential and k in tested) and k in expr) for k in energies.keys()]))

        essentialk = [cons[gene] for gene in regulated if gene in essential and gene in cons]
        nonessentialk = [cons[gene] for gene in regulated if gene not in essential and gene in tested and gene in cons]
        meankaks.append((mean(essentialk), mean(nonessentialk), sem(essentialk), sem(nonessentialk)))
        print >>outf, "KaKs: {nreggrowth} ({nrress} ess, {nrrnon} non)".format(
                      nnon=sum([(k not in essential and k in tested) for k in energies.keys()]),
                      nreggrowth=sum([k in cons for k in energies.keys()]), 
                      nrress=sum([(k in essential and k in cons) for k in energies.keys()]), 
                      nrrnon=sum([((k not in essential and k in tested) and k in cons) for k in energies.keys()]))

        essentialh = [h for gene, h in ham.iteritems() if gene in essential]
        nonessentialh = [h for gene, h in ham.iteritems() if gene not in essential and gene in tested ]
        meanham.append((mean(essentialh), mean(nonessentialh), sem(essentialh), sem(nonessentialh)))

        essentialdivE = [h for gene, h in divE.iteritems() if gene in essential]
        nonessentialdivE = [h for gene, h in divE.iteritems() if gene not in essential and gene in tested ]
        meandivE.append((mean(essentialdivE), mean(nonessentialdivE), sem(essentialdivE), sem(nonessentialdivE)))
        old_settings = seterr(divide='ignore', invalid='ignore')
        sqrtmeandivE.append((sqrt(meandivE[-1][0]), sqrt(meandivE[-1][1]), meandivE[-1][2]/(2*sqrt(meandivE[-1][0])), meandivE[-1][3]/(2*sqrt(meandivE[-1][1]))))
        seterr(**old_settings)

        print >>outf, ""
        print >>outf, "Mean Energy Difference:"
        epval = nullHypothesis(essentiale,  nonessentiale, outf)
        epvals.append(epval)
        print "Energy difference: {}, pval={}".format(mean(essentiale) - mean(nonessentiale), epval)

        print >>outf, ""
        print >>outf, "Variance Energy Difference:"
        vpval = nullHypothesis(essentiale,  nonessentiale, outf, prop=var)
        print "Variance difference: {}, pval={}".format(var(essentiale) - var(nonessentiale), vpval)

        print >>outf, ""
        print >>outf, "Mean Hamming Distance:"
        hpval = nullHypothesis(essentialh,  nonessentialh, outf)
        hpvals.append(hpval)
        print "Hamming difference: {}, pval={}".format(mean(essentialh) - mean(nonessentialh), hpval)

        print >>outf, ""
        print >>outf, "Mean Cross-Species Squared Energy Distance:"
        depval = nullHypothesis(essentialdivE,  nonessentialdivE, outf)
        depvals.append(depval)
        print "Cross-species energy difference: {}, pval={}".format(mean(essentialdivE) - mean(nonessentialdivE), depval)

        outf.close()

    meane = array(meane)
    vare = array(vare)
    nseq = array(nseq)
    meanexpr = array(meanexpr)
    meankaks = array(meankaks)
    meanham = array(meanham)
    meandivE = array(meandivE)
    sqrtmeandivE = array(sqrtmeandivE)

    epvals = array(epvals)
    hpvals = array(hpvals)
    depvals = array(depvals)
    
    print ""
    print ""
    print "Mean Energy pval: {}".format(mean(epvals[~isnan(epvals)]))
    print "Mean Hamming pval: {}".format(mean(hpvals[~isnan(hpvals)]))
    print "Mean dE pval: {}".format(mean(depvals[~isnan(depvals)]))

    esstfs, unesstfs, untesttfs = loadEntropy(sys.argv[2],  essential, tested)

    fig = pylab.figure(figsize=(10,6))
    #fig.subplots_adjust(left=0.1, right=0.95, wspace=0.2)

    def bli(a,b):
        pylab.plot([a,b], [a,b], '-k')
        pylab.xlim(a,b)
        pylab.ylim(a,b)
    
    ecolor=(0.6,0.6,0.6)
    ms=6

    #pylab.title("Mean Energy")
    pylab.axes([0.1,0.6,0.225,0.35])
    pylab.text(-0.3,0.95,'A', fontsize=16, transform=pylab.gca().transAxes, family='serif')
    pylab.errorbar(meane[:,0], meane[:,1], meane[:,3], meane[:,2], '.', ecolor=ecolor, ms=ms)
    bli(-17,-5)
    pylab.xlabel(r'$\langle {E} \rangle_{\mathrm{essential}}$')
    pylab.ylabel(r'$\langle {E} \rangle_{\mathrm{nonessential}}$')
    

    old_settings = seterr(divide='ignore', invalid='ignore')

    #pylab.title("Variance")
    pylab.axes([0.425,0.6,0.225,0.35])
    pylab.text(-0.3,0.95,'B', fontsize=16, transform=pylab.gca().transAxes, family='serif')
    pylab.errorbar(vare[:,0], vare[:,1], fmt='.', ecolor=ecolor, ms=ms)
    bli(0,4)
    pylab.xticks([0,1,2,3,4])
    pylab.yticks([0,1,2,3,4])
    pylab.xlabel(r'$V_{\mathrm{essential}}$')
    pylab.ylabel(r'$V_{\mathrm{nonessential}}$')

    #pylab.title("Mean Hamming Distance")
    pylab.axes([0.75,0.6,0.225,0.35])
    pylab.text(-0.3,0.95,'C', fontsize=16, transform=pylab.gca().transAxes, family='serif')
    pylab.errorbar(log10(meanham[:,0]), log10(meanham[:,1]), meanham[:,3]/meanham[:,1]/log(10), meanham[:,2]/meanham[:,0]/log(10), '.', ecolor=ecolor, ms=ms)
    bli(-1.5,0.5)
    ticks = [-1,0]
    pylab.xticks(ticks, ['$10^{{{}}}$'.format(t) for t in ticks])
    pylab.yticks(ticks, ['$10^{{{}}}$'.format(t) for t in ticks])
    pylab.xlabel(r'$ \langle {d} \rangle_{\mathrm{essential}}$')
    pylab.ylabel(r'$ \langle {d} \rangle_{\mathrm{nonessential}}$')
    ##COMMENTED: code which produces a linear-scale plot of hamming distances
    #pylab.errorbar(meanham[:,0], meanham[:,1], meanham[:,3], meanham[:,2], '.', ecolor=ecolor, ms=ms)
    #bli(0,1.5)
    #pylab.xticks([0,0.5,1,1.5])
    #pylab.yticks([0,0.5,1,1.5])

    #pylab.title("Mean dE^2 Distance")
    pylab.axes([0.255,0.1,0.225,0.35])
    pylab.text(-0.3,0.95,'D', fontsize=16, transform=pylab.gca().transAxes, family='serif')
    pylab.errorbar(log10(meandivE[:,0]), log10(meandivE[:,1]), meandivE[:,3]/meandivE[:,1]/log(10), meandivE[:,2]/meandivE[:,0]/log(10), '.', ecolor=ecolor, ms=ms)
    bli(-2.5,1)
    ticks = [-2,-1,0,1]
    pylab.xticks(ticks, ['$10^{{{}}}$'.format(t) for t in ticks])
    pylab.yticks(ticks, ['$10^{{{}}}$'.format(t) for t in ticks])
    pylab.xlabel(r'${ \langle \Delta E^2 \rangle}_{\mathrm{essential}}$')
    pylab.ylabel(r'${ \langle \Delta E^2 \rangle}_{\mathrm{nonessential}}$')
    ##COMMENTED: code which produces a plot of sqrt <dE^2>
    #pylab.errorbar(sqrtmeandivE[:,0], sqrtmeandivE[:,1], sqrtmeandivE[:,3], sqrtmeandivE[:,2], '.', ecolor=ecolor, ms=ms)
    #bli(0,2.3)
    #pylab.xticks([0,1.0,2.0])
    #pylab.yticks([0,1.0,2.0])
    #pylab.xlabel(r'$\sqrt{ \langle \Delta E^2 \rangle}_{\mathrm{essential}}$')
    #pylab.ylabel(r'$\sqrt{ \langle \Delta E^2 \rangle}_{\mathrm{nonessential}}$')

    #pylab.title('Sequence Entropy')
    pylab.axes([0.6,0.1,0.225,0.35])
    pylab.text(-0.3,0.95,'E', fontsize=16, transform=pylab.gca().transAxes, family='serif')
    print "TF site entropy:"
    print "Ess TF: {}\tNoness TF: {}\tUnknown: {}\n".format(len(esstfs), len(unesstfs), len(untesttfs))
    print  nullHypothesis(esstfs, unesstfs)
    smin, smax = np.min(esstfs + unesstfs), np.max(esstfs + unesstfs)
    bins = arange(smin, smax+1e-5, (smax-smin)/10.0)
    pylab.hist([unesstfs, esstfs], bins=bins, normed=True)
    pylab.legend(['nonessential', 'essential'], loc='upper left', fontsize=10)
    pylab.xlabel('Sequence Entropy')
    pylab.ylabel('Normalized Counts')
    pylab.xticks([0.6, 1.3, 2.0])
    pylab.yticks([0, 2, 4])

    #pylab.tight_layout()
    
    pylab.show()
    
if __name__ == '__main__':
    main()

