#!/usr/bin/env python2
from __future__ import with_statement
from scipy import *
from numpy import min, max
import numpy as np
import pylab
import sys, os, re, glob
import load, getChi
from load import getFreqs
import scipy.stats 
from essentiality import seqLoad, getEnergy, hamming, loadSites, loadEssentialityData

cc = scipy.stats.spearmanr
sem = lambda x: scipy.stats.sem(x) if len(x) > 1 else nan
mean = lambda x: np.mean(x) if len(x) > 0 else nan
var = lambda x: np.var(x) if len(x) > 0 else nan

outdir = 'results'

def main():
    pylab.rcParams['figure.subplot.bottom'] = 0.15

    essential, tested, rates, expr, cons = loadEssentialityData()
    
    growthcorrs = []
    exprcorrs = []
    kakscorrs = []
    tfs = []
    for emfile in glob.glob(os.path.join(sys.argv[1], '*')):
    #for emfile in [os.path.join(sys.argv[1], 'REB1')]:
        em = loadtxt(emfile)
        tf = os.path.basename(emfile)
        tfs.append(tf)
        tffile = os.path.join(sys.argv[2], tf)
        if not os.path.exists(tffile):
            print "skipping" + tffile
            continue

        #load gene to  TFBS map
        regsites = loadSites(tffile)
        
        energies = dict([(gene, getEnergy(em, sites['recomputed'].strip())) for gene, pos, dist, sites in regsites])
        ham = dict([(gene, hamming(sites['cerevisiae'], sites['paradoxus'])) for gene, pos, dist, sites in regsites if 'paradoxus' in sites])
        alle = [e for gene, e in energies.iteritems()]

        regulated = energies.keys()
        if mean(alle) > -5: 
            print "Skipping", tf
            continue

        try:
            outfile = open(os.path.join(outdir, tf, 'ecorr.txt'), 'wt')
        except:
            continue

        pylab.figure()
        growths = array([(energies[gene], rates[gene]) for gene in regulated if gene in rates])
        print >>outfile, "Growthrate:  corr={: .3f},  p={: .3f}".format(*cc(growths[:,0], growths[:,1]))
        growthcorrs.append(cc(growths[:,0], growths[:,1])[1])
        pylab.plot(growths[:,0], growths[:,1], '.')
        pylab.xlabel('Energy')
        pylab.ylabel('Growth Rate')
        pylab.tight_layout()
        pylab.savefig(os.path.join(outdir, tf, 'growthVe.svg'))

        pylab.figure()
        rexpr = array([(energies[gene], expr[gene]) for gene in regulated if gene in expr])
        print >>outfile, "Expression:  corr={: .3f},  p={: .3f}".format(*cc(rexpr[:,0], rexpr[:,1]))
        exprcorrs.append(cc(rexpr[:,0], rexpr[:,1])[1])
        pylab.plot(rexpr[:,0], rexpr[:,1], '.')
        pylab.xlabel('Energy')
        pylab.ylabel('Expression')
        pylab.tight_layout()
        pylab.savefig(os.path.join(outdir, tf, 'exprVe.svg'))

        pylab.figure()
        kaks = array([(energies[gene], cons[gene]) for gene in regulated if gene in cons])
        print >>outfile, "Ka/Ks:       corr={: .3f},  p={: .3f}".format(*cc(kaks[:,0], kaks[:,1]))
        kakscorrs.append(cc(kaks[:,0], kaks[:,1])[1])
        pylab.plot(kaks[:,0], kaks[:,1], '.')
        pylab.xlabel('Energy')
        pylab.ylabel('$K_A/K_S$')
        pylab.tight_layout()
        pylab.savefig(os.path.join(outdir, tf, 'kaksVe.svg'))

        #pylab.show()
        pylab.close('all')

        outfile.close()
    
    growthcorrs = array(growthcorrs)
    exprcorrs = array(exprcorrs)
    kakscorrs = array(kakscorrs)
    
    print "Growth pvals:"
    print growthcorrs
    print "Growth pvals < 0.05:"
    print [(tfs[i],growthcorrs[i]) for i in argwhere(growthcorrs<0.05)]
    print "Mean Growth Pval:", mean(growthcorrs)
    print ""
    print "Expr pvals:"
    print exprcorrs
    print "Expr pvals < 0.05:"
    print [(tfs[i],exprcorrs[i]) for i in argwhere(exprcorrs<0.05)]
    print "Mean Expr Pval:", mean(exprcorrs)
    print ""
    print "KaKs pvals:"
    print kakscorrs
    print "KaKs pvals < 0.05:"
    print [(tfs[i],kakscorrs[i]) for i in argwhere(kakscorrs<0.05)]
    print "Mean KaKs pval:", mean(kakscorrs)
    
if __name__ == '__main__':
    main()

