#!/usr/bin/env python2
from __future__ import with_statement
from scipy import *
import numpy as np
import pylab
import sys, os, re, glob
import load, getChi
from load import getFreqs
import scipy.stats
from numpy.random import choice as randChoice

class simpleLoader:
    def __init__(self, names):
        self.nucNums = -ones(256, int) 
        self.nucNums[frombuffer(names, uint8)] = arange(len(names))

    def __call__(self, seq):
        return self.nucNums[frombuffer(seq, uint8)]
seqLoad = simpleLoader('ACGT')

def hamming(str1,str2):
    diffs = 0
    for ch1, ch2 in zip(str1, str2):
        if ch1 != ch2:
            diffs += 1
    return diffs

def getEnergy(em, site):    
    seq = seqLoad(site)
    seqlen = len(seq)
    return sum(em[arange(seqlen,dtype=int), seq])

def loadSites(fn):
    with open(fn) as f:
        lines = f.readlines()    
    
    regulated = []
    for l in lines:
        parts = l.split()
        gene, pos, dist = parts[:3]
        seqs = eval(" ".join(parts[3:]))
        regulated.append((gene, pos, dist, seqs))
    return regulated

def loadEssentialityData():
    with open('../rawData/Deletion/Regression_Tc1_hom.txt') as f:
        data = [l.split('\t') for l in f.readlines()]
        #gene, orf, no. of probes, measurments, F-statistic, -log(p-value), difference, YPD, YPDGE, YPG, YPE, YPL
        rates = dict([(d[0], min([float(x) for x in d[7:12]])) for d in data[1:] if len(d) > 5 and not d[-1].isspace()])
        rates.update(dict([(d[1], min([float(x) for x in d[7:12]])) for d in data[1:] if len(d) > 5 and not d[-1].isspace()]))
        essential = set([d[0] for d in data[1:] if not (len(d) > 5 and not d[-1].isspace())])
        essential.update([d[1] for d in data[1:] if not (len(d) > 5 and not d[-1].isspace())])
        tested = set([d[0] for d in data[1:]])
        tested.update([d[1] for d in data[1:]])

    with open('../rawData/transcriptome') as f:
        expr = [l.split('\t') for l in f.readlines()]
        expr = dict([(g,float(e)) for g,e in expr if e != 'nan\n'])
    with open('../paradoxus/dSdN.txt') as f:
        cons = [l.split() for l in f.readlines()]
        cons = dict([(l[0],float(l[2])) for l in cons if float(l[2]) < 2])
    with open('../rawData/genenames') as f:
        def updateData(d, altnames):
            k = set(d.keys())
            matches = ((k & names, names) for names in altnames)
            matches = ((d[hits.pop()], names) for hits, names in matches if hits)
            d.update(dict([(n, v) for v,names in matches for n in names]))

        altnames = [set(l.split()) for l in f.readlines()]
        for names in altnames:
            if essential & names:
                essential.update(names)
            if tested & names:
                tested.update(names)
        updateData(rates, altnames)
        updateData(expr, altnames)
        updateData(cons, altnames)

    return essential, tested, rates, expr, cons

