#!/usr/bin/env python2
import glob, sys, os, shutil, re
from scipy import *
import load
import pylab
import latex2svg

datadir = sys.argv[1]

def writelatex(tex):
    tex = '${}$'.format(tex)
    fn = latex2svg.svgfilename(latex2svg.latexhash(tex))
    if not os.path.exists('lateximg'):
        os.mkdir('lateximg')
    fp = os.path.join('lateximg', fn)
    if not os.path.exists(fp):
        svg = latex2svg.makeLatex(tex)
        with open(fp, 'wt') as f:
            f.write(svg)
    else:
        with open(fp, 'rt') as f:
            svg = f.read()
    return svg.strip()
rhotxt = writelatex(r'\rho')
ebar = writelatex(r'\langle {E}\rangle')
ebar_e = writelatex(r'\langle {E}\rangle_{\mathbf{ess}}')
ebar_n = writelatex(r'\langle {E}\rangle_{\mathbf{non}}')
debar = writelatex(r'\langle \Delta {E}^2\rangle ')
debar_e = writelatex(r'\langle \Delta {E}^2\rangle_{\mathbf{ess}}')
debar_n = writelatex(r'\langle \Delta {E}^2\rangle_{\mathbf{non}}')
hbar = writelatex(r'\langle {d}\rangle ')
hbar_e = writelatex(r'\langle {d}\rangle_{\mathbf{ess}}')
hbar_n = writelatex(r'\langle {d}\rangle_{\mathbf{non}}')
V = writelatex(r'V')
V_n = writelatex(r'V_{\mathbf{non}}')
V_n = writelatex(r'V_{\mathbf{non}}')
kaks_tex = writelatex(r'K_A/K_S')
p_tex = writelatex(r'p')

tfdirs = [os.path.join(datadir, tf) for tf in os.walk(datadir).next()[1]]

#sort them by nUniqueSeq
nSeqs = {}
for dr in tfdirs:
    tf = os.path.basename(dr)
    with open(os.path.join('../rawData/stats0.9', tf, 'stats.txt')) as f:
        lines = f.readlines()
        nSeqs[dr] = (int(lines[0].split()[1]), int(lines[1].split()[1]))
#print >>sys.stderr, tfdirs, [nSeqs[dr] for dr in tfdirs]
tfdirs.sort(key=lambda x: nSeqs[x], reverse=True)
def cc(a,b):
    c = cmp(nSeqs[a][1], nSeqs[b][1])
    if c != 0:
        return c
    return cmp(nSeqs[a][0], nSeqs[b][0])
tfdirs.sort(cmp=cc, reverse=True)
#tfdirs = tfdirs[:12]

def div(*e, **ks):
    return "<div {}>{}</div>".format(ks.get('attr', ""), "".join(map(str, e)))
def td(*e, **ks):
    return "<td {}>{}</td>".format(ks.get('attr', ""), "".join(map(str, e)))
def th(*e):
    return "<th>{}</th>".format("".join(map(str, e)))
def tr(*e, **ks):
    return "<tr {}>{}</tr>".format(ks.get('attr', ""), "".join(map(str, e)))
def table(*e, **ks):
    return "<table {}>{}</table>".format(ks.get('attr', ""), "".join(map(str, e)))


def fmtfloat(s):
    f = float(s)
    return "{:.3f}".format(f) if not isnan(f) else '&mdash;'

dat = []
def printRow(tfdir):
    tf = os.path.basename(tfdir)
    
    try:
        with open(os.path.join('../rawData/stats0.9/', tf, 'stats.txt')) as f:
            nSeq, nUnique, emEntr, siteEntropy, kldiff = [l.split()[1] for l in f.readlines()]
    except:
        raise Exception(tf)
        nSeq = nUnique = emEntr = siteEntropy = kldiff = None
        
    try:
        with open(os.path.join('../rawData/stats0.9/', tf, 'essinfo.txt')) as f:
            txt = f.readlines()
            isEssential = txt[0].split()[2] == 'essential'
            nRegEssential = txt[1].split()[0]
            nRegNonessential = txt[2].split()[0]
            nRegUnknown = txt[3].split()[0]
        esstxt = "Genes Regulated:<BR>{} essential<BR>{} nonessential<BR>{} unknown".format(
                  nRegEssential, nRegNonessential, nRegUnknown)

    except:
        raise Exception(tf)
        print "AAAHH", tfdir
    
    with open(os.path.join(tfdir, 'ecorr.txt')) as f:
        corr = [re.match('([\w/]+):\s+corr=([ \d\.-]+),\s+p=([ \d\.-]+)', l).groups() for l in f.readlines()]
        corr = dict((n, (float(r),float(p))) for n,r,p in corr)
    with open(os.path.join(tfdir, 'essnoness.txt')) as f:
        essnontxt = f.readlines()

        nexpr, nexpress, nexprnon = [int(x) for x in re.match('Expr: (\d+) \((\d+) ess, (\d+) non\)', essnontxt[1]).groups()]
        nkaks, nkaksess, nkaksnon = [int(x) for x in re.match('KaKs: (\d+) \((\d+) ess, (\d+) non\)', essnontxt[2]).groups()]
        
        eEss, eNon = [fmtfloat(x) for x in re.match('Ess:\s+([^\s]+)\s+Non:\s+([^\s]+)', essnontxt[5]).groups()]
        eDiffp = fmtfloat(essnontxt[6].split()[1])

        vEss, vNon = [fmtfloat(x) for x in re.match('Ess:\s+([^\s]+)\s+Non:\s+([^\s]+)', essnontxt[9]).groups()]
        vDiffp = fmtfloat(essnontxt[10].split()[1])

        hEss, hNon = [fmtfloat(x) for x in re.match('Ess:\s+([^\s]+)\s+Non:\s+([^\s]+)', essnontxt[13]).groups()]
        hDiffp = fmtfloat(essnontxt[14].split()[1])

        deEss, deNon = [fmtfloat(x) for x in re.match('Ess:\s+([^\s]+)\s+Non:\s+([^\s]+)', essnontxt[17]).groups()]
        deDiffp = fmtfloat(essnontxt[18].split()[1])

    with open(os.path.join(tfdir, 'tssdist.txt')) as f:
        tsscorr = [float(x) for x in f.readlines()[0].split()]
        corr['tss'] = (tsscorr[0], tsscorr[1])
    
    infotable = table(tr(th(''), th('Essential'), th('Noness.'), th(p_tex)),
                      tr(td('Total&nbsp;Data'), td(nRegEssential), td(nRegNonessential), td("")),
                      tr(td('Expr&nbsp;Data'), td(nexpress), td(nexprnon), td("")),
                      tr(td('<i>S. Par.</i>&nbsp;Data'), td(nkaksess), td(nkaksnon), td("")),
                      tr(td(ebar), td(eEss), td(eNon), td(eDiffp), attr='style="border-top: 1px solid black"'),
                      tr(td(V), td(vEss), td(vNon), td(vDiffp)),#td("&mdash;")), 
                      tr(td(debar), td(deEss), td(deNon), td(deDiffp)),
                      tr(td(hbar), td(hEss), td(hNon), td(hDiffp)),
                     attr='class="params"')

    tftxt = div(tf + "&nbsp;({}&nbsp;TF)<BR><BR>".format('essential' if isEssential else 'nonessential') + \
                 "Total sites: {}<BR>Unique sites: {}<BR><BR>{}".format(nSeq, nUnique, infotable), attr='class="infotxt"')

    return tr(td(tftxt), 
             td('{} = {:.3f}, {} = {:.3f} <img height="250px" src="results/{}/growthVe.svg">'.format(rhotxt, corr['Growthrate'][0], p_tex, corr['Growthrate'][1], tf)),
             td('{} = {:.3f}, {} = {:.3f} <img height="250px" src="results/{}/exprVe.svg">'.format(rhotxt, corr['Expression'][0], p_tex, corr['Expression'][1], tf)),
             td('{} = {:.3f}, {} = {:.3f} <img height="250px" src="results/{}/kaksVe.svg">'.format(rhotxt, corr['Ka/Ks'][0], p_tex, corr['Ka/Ks'][1], tf)),
             td('{} = {:.3f}, {} = {:.3f} <img height="250px" src="results/{}/tssVe.svg">'.format(rhotxt, corr['tss'][0], p_tex, corr['tss'][1],  tf))
            )

print """<!DOCTYPE html>
<html>
<head>
<style>
img{
}
td{
    border-width: 1px;
    border-style: solid;
    padding-bottom: 30px;
}
table {
    border:none;
    border-collapse: collapse;
    page-break-inside:auto;
}

tr{ 
    page-break-inside:avoid; 
    page-break-after:auto;
}

table td{
    border-left: none;
    border-right: none;
    text-align: center;
}

table td.firstcol{
    text-align: left;
}

table.params{
    margin: 10px;
}

table.params td{
    border: none;
    border-right: solid;
    border-width: 1px;
    padding: 1px 5px 1px 20px;
    text-align: right;
}

table.params th{
    border: none;
    border-right: solid;
    border-bottom: solid;
    border-width: 1px;
    text-align: right;
    padding: 1px 5px 1px 20px;
}

.infotxt{
    width: 450px;
    text-align: left;
}


</style>
</head>
<body>

"""

header =  tr(th('(A) Dataset'), 
             th('(B) Growth Rate'),
             th('(C) Expression Level'), 
             th('(D) {} Ratio'.format(kaks_tex)), 
             th('(E) TSS Distance'), 
            )

print table(*([header] + [printRow(fn) for fn in tfdirs]))

print """
</body>
</html>
"""
