#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "logLike.h"

int nparam = 1;

//only set once at program start
static int nx;
static double *x;
static double *background;
static double *logbackground;

static int nEn;
static double *energies;
static double *seqbgp;
static int *enBins;
static double sumlogseqbgp;

static double maxe;

//temporary storage, is updated frequently
static double *logFitnesses;
static double *normterms;

void init(int nx_i, double *x_i, double *background_i, int nEn_i, double *energies_i, double *bgp_i){
    int i, j;

    nx = nx_i;
    x = x_i;
    background = background_i;

    nEn = nEn_i; 
    energies = energies_i;
    seqbgp = bgp_i;

    double dx = x[1]-x[0];
    maxe = x[nx-1];

    logFitnesses = malloc(sizeof(double)*nx);
    normterms = malloc(sizeof(double)*nx);

    //figure out which bin each energy is in
    enBins = malloc(sizeof(int)*nEn);
    for(i = 0; i < nEn; i++){
        j = 0;
        while(j < nx && energies[i] > x[j] + dx/2){ //just do a linear search
            j++;
        }
        enBins[i] = j;
    }

    for(i = 0; i < nEn; i++){
        sumlogseqbgp += log(seqbgp[i]);
    }

    //precalculate logBackSum,  used in the norm calc
    logbackground = malloc(sizeof(double)*nx);
    for(i = 0; i < nx; i++){
        logbackground[i] = log(background[i]);
    }
}

double logFitness(double a,  double en){
    return -a*en;
}

void recalcLogFitnesses(double a){
    int i;
    for(i = 0; i < nx; i++){
        logFitnesses[i] = logFitness(a,x[i]);
    }
}

double logProbNorm(double a){
    int i;

    double maxterm = -INFINITY;
    for(i = 0; i < nx; i++){
        normterms[i] = logbackground[i] + logFitnesses[i];
        if(normterms[i] > maxterm){
            maxterm = normterms[i];
        }
    }

    double sum = 0;
    for(i = 0; i < nx; i++){
        sum += exp(normterms[i] - maxterm);
    }
    return maxterm + log(sum);
}

void mapParam(gsl_vector *v){
    gsl_vector_set(v, 0, log(gsl_vector_get(v,0)));
}

void unmapParam(gsl_vector *v){
    gsl_vector_set(v, 0, exp(gsl_vector_get(v,0)));
}

double logLikelihood(const gsl_vector *v){
    double a = exp(gsl_vector_get(v,0));

    recalcLogFitnesses(a);
    double distlognorm = logProbNorm(a);

    double sum = 0;
    int i;
    for(i = 0; i < nEn; i++){
        sum +=  logFitness(a, energies[i]);
    }
    return sumlogseqbgp + sum - nEn*distlognorm;
}

double fitness(double en, const gsl_vector *v){
    double a = exp(gsl_vector_get(v,0));

    return exp(logFitness(a, en));
}

void printLikelihoodVars(){
    printf("exp(a)");
}
