#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "logLike.h"

int nparam = 3;

//only set once at program start
static int nx;
static double *x;
static double *background;
static double *logbackground;

static int nEn;
static double *energies;
static double *seqbgp;
static int *enBins;
static double sumlogseqbgp;

//temporary storage, is updated frequently
static double *NlogFitnesses;
static double *normterms;

double logaddexp(double x, double y){
    double diff = x-y;
    if(diff > 0){
        return x + log1p(exp(-diff));
    }
    else if(diff <= 0){
        return y + log1p(exp(diff));
    }
    else{//nan or inf
        return x+y;
    }
}

void init(int nx_i, double *x_i, double *background_i, int nEn_i, double *energies_i, double *bgp_i){
    int i,j;

    nx = nx_i;
    x = x_i;
    background = background_i;

    nEn = nEn_i; 
    energies = energies_i;
    seqbgp = bgp_i;

    double dx = x[1]-x[0];

    NlogFitnesses = malloc(sizeof(double)*nx);
    normterms = malloc(sizeof(double)*nx);

    //figure out which bin each energy is in
    enBins = malloc(sizeof(int)*nEn);
    for(i = 0; i < nEn; i++){
        j = 0;
        while(j < nx && energies[i] > x[j] + dx/2){ //just do a linear search
            j++;
        }
        enBins[i] = j;
    }

    for(i = 0; i < nEn; i++){
        sumlogseqbgp += log(seqbgp[i]);
    }

    //precalculate logBackSum,  used in the norm calc
    logbackground = malloc(sizeof(double)*nx);
    for(i = 0; i < nx; i++){
        logbackground[i] = log(background[i]);
    }
}

double logFitness(double mu, double B, double c, double en){
    return logaddexp(B*(en-mu)+log(c),0) - logaddexp(B*(en-mu),0);
}

void recalcNLogFitnesses(double N, double mu, double B, double c){
    int i;
    for(i = 0; i < nx; i++){
        NlogFitnesses[i] = N*logFitness(mu,B,c,x[i]);
    }
}

double logProbNorm(double N, double mu, double B, double c){
    int i;

    double maxterm = -INFINITY;
    for(i = 0; i < nx; i++){
        normterms[i] = logbackground[i] + NlogFitnesses[i];
        //normterms[i] = logbackground[i] + N*logFitness(mu,B,c,x[i]);
        if(normterms[i] > maxterm){
            maxterm = normterms[i];
        }
    }

    double sum = 0;
    for(i = 0; i < nx; i++){
        sum += exp(normterms[i] - maxterm);
    }
    return maxterm + log(sum);
}

void mapParam(gsl_vector *v){
    gsl_vector_set(v, 0, log(gsl_vector_get(v,0)));
}

void unmapParam(gsl_vector *v){
    gsl_vector_set(v, 0, exp(gsl_vector_get(v,0)));
}

double logLikelihood(const gsl_vector *v){
    double N = exp(gsl_vector_get(v,0));
    double mu = gsl_vector_get(v,1);
    double B = gsl_vector_get(v,2);
    double c = 0.99;

    recalcNLogFitnesses(N,mu,B,c);
    double distlognorm = logProbNorm(N,mu,B,c);

    double sum = 0;
    int i;
    for(i = 0; i < nEn; i++){
        //sum +=  NlogFitnesses[enBins[i]];
        sum +=  N*logFitness(mu,B,c,energies[i]);
    }
    //printf("%g %g %g\n", sumlogseqbgp , sum , distlognorm);
    return sumlogseqbgp + sum - nEn*distlognorm;
}

double fitness(double en, const gsl_vector *v){
    double N = exp(gsl_vector_get(v,0));
    double mu = gsl_vector_get(v,1);
    double B = gsl_vector_get(v,2);
    double c = 0.99;

    return exp(logFitness(mu, B, c, en));
}

void printLikelihoodVars(){
    printf("N mu B");
}
