#!/usr/bin/env python2
import os, sys
from scipy import *
import numpy as np
from getDist import getEDist, resampleDist, getEs, getEBins, getSeqProb
from load import loadSites, getEnergies

def getIntergenic(intergenicf):
    #get the (di)nucleotide prob from the intergenic data
    nBases = 4
    intergenic = loadSites(intergenicf, 'ACGT')[0].flatten()
    pa = histogram(intergenic, bins=[0,1,2,3,4])[0]
    pa = pa/float(sum(pa))
    dnuc = intergenic[:-1]*nBases + intergenic[1:]
    dpa = histogram(dnuc, bins=arange(nBases**2+1))[0].astype(float)
    dpac = dpa/repeat(sum(dpa.reshape(nBases,nBases),1),nBases) #transition prob
    return pa,dpac

def getDist(seq, em, pa, dpac,nbins):
    seqen = getEnergies(seq, em)
    x,back = resampleDist(getEDist(em,nbins,16, pa, dpac),16)
    return seqen, (x,back)

def main():
    pa,dpac = getIntergenic(sys.argv[1])
    seq, info = loadSites(sys.argv[2], 'ACGT')
    em = loadtxt(sys.argv[3])

    seqen = getEnergies(seq, em)
    seqbgp = getSeqProb(seq, pa, dpac)

    if len(sys.argv) == 5:
        nbins = int(sys.argv[4])
    else:
        mine = sum(np.min(em,1))  #store the old mine to unshift at the end
        maxe = sum(np.max(em,1))

        de = (max(seqen) - min(seqen))/15.0
        nbins = int( (maxe-mine)/de )
    
    #chop off irrelevant rows, to avoid memory problems
    em = em[sum(em==0,axis=1) != 4]
        
    x,dx,xbins = getEBins(em, nbins)
    es,ws = getEs(em, pa, dpac)
    hn,xbins = histogram(es, bins=xbins, weights=ws)

    savetxt(sys.stdout, [seqen])
    savetxt(sys.stdout, [seqbgp])
    savetxt(sys.stdout, c_[x,hn])

if __name__ == '__main__':
    main()

