#!/usr/bin/env python2
import sys, re, glob, string, os
import geneParsers

def revcomplement(seq):
    return complement(seq)[::-1]

def complement(seq):
    return seq.translate(string.maketrans('ACGT', 'TGCA'))

def removeLastCodon(seq):
    return re.sub('[ACGT](-*)[ACGT](-*)[ACGT](-*)$', r'\1\2\3', seq)

outdir = 'stripped'
blacklist = ['YER109C.fasta', 'YOR031W.fasta']

for fn in glob.glob('alignments/*'):
#for fn in glob.glob('alignments/YPR198W.fasta'):
    if os.path.basename(fn) in blacklist:
        print >>sys.stderr, "Blacklisted!"
        continue

    fastaseq = dict((seq.seqID, seq.seq) for seq in geneParsers.loadFASTA(fn))

    Cer = fastaseq['Cer']
    Par = fastaseq['Par']
    seqs = [Cer, Par]

    #skip sequences which are too unequal
    lengths = [len(re.sub('[^ACGTN]', "", seq)) for seq in seqs]
    if float(min(lengths))/max(lengths) < 0.8:
        print >>sys.stderr, "Mismatched!", lengths, float(min(lengths))/max(lengths)
        continue
    ##remove all single or double gaps except for leading gaps
    #seqs = [re.sub('(?<!$)-*', '', seq) for seq in seqs]

    #remove gaps that cause frame shift
    seqs = [re.sub('((?:---)*)-?-?', r'\1', seq) for seq in seqs]
    #remove trailing gaps (but will add them back)
    seqs = [re.sub('-*$', '', seq) for seq in seqs]
    #readjust trailing gaps so sequences are equal length
    if len(seqs[0]) < len(seqs[1]):
        seqs[0] = seqs[0] + '-'*(len(seqs[1]) - len(seqs[0]))
    if len(seqs[1]) < len(seqs[0]):
        seqs[1] = seqs[1] + '-'*(len(seqs[0]) - len(seqs[1]))
    if len(seqs[0])%3 != 0:
        print >>sys.stderr, "Not mutiple of 3!", os.path.basename(fn)

    #start = re.search('[^-]', Cer).start()
    #end = len(Cer) - re.search('[^-]', Cer[::-1]).start()
    #print fn
    ##if name[-1] == 'W':
    ##    seqs = [seq[start:end] for seq in seqs]
    ##elif name[-1] == 'C':
    ##    seqs = [revcomplement(seq[start:end]) for seq in seqs]
    ##else:
    ##    print >>sys.stderr, "Not W or C!"
    ##    continue

    name,v = re.match('alignments/([^-]*)(-[A-Z])?\.fasta', fn).groups()
    with open(os.path.join(outdir, name + (v or "")), 'wt') as f:
        #print >>f, "   {} {}".format(2, len(seqs[0]))
        #print >>f, "Cer\n{}".format(seqs[0])
        #print >>f, "Par\n{}".format(seqs[1])
        print >>f, "   {} {}".format(2, len(seqs[0])-3)
        print >>f, "Cer\n{}".format(removeLastCodon(seqs[0])) #remove stop codon
        print >>f, "Par\n{}".format(removeLastCodon(seqs[1]))
        print >>f, ""
