#!/usr/bin/env python2
from __future__ import with_statement 
from scipy import empty, array, recarray, select
import re, string


def revcomplement(seq):
    return complement(seq)[::-1]

def complement(seq):
    return seq.translate(string.maketrans('ACGT', 'TGCA'))

class FASTAseq:
    def __init__(self, seqID, modifiers, description, seq):
        self.seqID = seqID
        self.modifiers = modifiers
        self.description = description
        self.seq = seq

def loadFASTA(file):
    f = open(file, 'rt')
    
    seqIDpat = re.compile("^>\s*([^\s\[\]]+)\s+")
    modifierPat = re.compile("\s*\[([^=]*)=([^=]*)\]")
    def parseHeader(header):
        m = seqIDpat.match(header)
        id = m.groups()[0]

        pos = m.end()
        modifiers = {}
        while(True):
            m = modifierPat.match(header, pos)
            if(m == None):
                break
            g = m.groups()
            modifiers[g[0]] = g[1]
            pos = m.end()

        description = header[pos:].strip()
        
        return id, modifiers, description
    
    sequences = []
    header = None
    for line in f:
        if(line.isspace()):
            continue

        if(line[0] == '>'): #seq header
            if(header != None):
                sequences.append( FASTAseq(header[0], header[1], header[2], ''.join(seqParts)) )
            header = parseHeader(line)
            seqParts = []
        else:
            seqParts.append(line.strip())
    sequences.append( FASTAseq(header[0], header[1], header[2], ''.join(seqParts)) )
    
    return sequences

def loadGFF(file):
    f = open(file, 'rt')
    
    d = [('seqname', 'O'), ('source', 'O'), ('feature', 'O'), 
         ('start', 'i'), ('end', 'i'), ('score', 'd'), 
         ('strand', 'S1'), ('frame', 'S1'), ('attributes', 'O')]
    
    lines = f.readlines()
    data = recarray(len(lines), dtype=d)
    pat = re.compile("^" + "(\S+)\s+"*7 + "(\S+)\s+(.*)#?.*")

    for i in range(len(lines)):
        match = pat.match(lines[i])
        if(match):
            data[i] = match.groups()
        else:
            raise IOError("invalid GFF file at line %d" % i)
    
    data['start'] -= 1 #for python convenience later. warning, not to spec!
    #warning: data.end is one past the last element, so [start:end] works.
    f.close()
    return data

def loadGenome(dirname):
    chromosomes = []
    for i in range(1,17):
        chromosomes.append( loadFASTA('%s/chr%02d.fsa' % (dirname, i))[0] )
    return chromosomes

def gff2seq(gff, genome):
    sequences = []
    for seq in gff:
        ind = [s.modifiers['chromosome'] for s in genome].index(seq.seqname)
        if(seq.strand == '+'):
            subseq = revcomplement(genome[ind].seq[seq.start+1:seq.end+1])
        else:
            subseq = genome[ind].seq[seq.start:seq.end]
        sequences.append(subseq)
    
    return sequences

def main():
    d = loadGFF('macisaac/p001_c3.gff')
    pat = re.compile('ABF1')
    d = d[ array([pat.search(a) != None for a in d.attributes]) ]
    g = loadGenome('serGenome')
    seqs = gff2seq(d, g)
    for s in seqs:
        print s

if __name__ == '__main__':
    main()

