#!/usr/bin/env python2
import glob, sys, os, shutil, re
from scipy import *
import load
import pylab
import latex2svg

def printSci(x):
    return "%.3g" % x
    #if x < 1000:
    #    return "{:.3f}".format(x)
    e = int(log10(x))
    return r"{:.2f} &#xd7; 10<sup>{{{:d}}}</sup>".format(x / 10**e, e)

def writelatex(tex):
    tex = '${}$'.format(tex)
    fn = latex2svg.svgfilename(latex2svg.latexhash(tex))
    if not os.path.exists('lateximg'):
        os.mkdir('lateximg')
    fp = os.path.join('lateximg', fn)
    if not os.path.exists(fp):
        svg = latex2svg.makeLatex(tex)
        with open(fp, 'wt') as f:
            f.write(svg)
    else:
        with open(fp, 'rt') as f:
            svg = f.read()
    return svg.strip()

paramNames = [writelatex(x) for x in [r'\nu', r'\mu', r'\beta', 'f_0']]
lognutxt, logbetatxt, Tf0txt = [writelatex(x) for x in [r'\log \nu', r'\log \beta', '\mathcal{T}(f_0)']]
nutxt, mutxt, betatxt, f0txt = paramNames
gammatxt = writelatex(r'\gamma')
alphatxt = writelatex(r'\alpha')
lltxt = writelatex(r'\log \mathcal{L}')
lambdatxt = writelatex(r'\lambda')

datadir = sys.argv[1]

tfdirs = [os.path.join(datadir, tf) for tf in os.walk(datadir).next()[1]]

#sort them by nUniqueSeq
nSeqs = {}
for tfdir in tfdirs:
    tf = os.path.basename(tfdir)
    with open(os.path.join('../rawData/stats0.9/', tf, 'stats.txt')) as f:
        lines = f.readlines()
        nSeqs[tfdir] = (int(lines[0].split()[1]), int(lines[1].split()[1]))
#print >>sys.stderr, tfdirs, [nSeqs[dr] for dr in tfdirs]
tfdirs.sort(key=lambda x: nSeqs[x], reverse=True)
def cc(a,b):
    c = cmp(nSeqs[a][1], nSeqs[b][1])
    if c != 0:
        return c
    return cmp(nSeqs[a][0], nSeqs[b][0])
tfdirs.sort(cmp=cc, reverse=True)
tfdirs = tfdirs[:12]
#print >>sys.stderr, tfdirs, [nSeqs[dr] for dr in tfdirs]

def div(*e, **ks):
    return "<div {}>{}</div>".format(ks.get('attr', ""), "".join(map(str, e)))
def td(*e, **ks):
    return "<td {}>{}</td>".format(ks.get('attr', ""), "".join(map(str, e)))
def th(*e):
    return "<th>{}</th>".format("".join(map(str, e)))
def tr(*e):
    return "<tr>{}</tr>".format("".join(map(str, e)))
def table(*e, **ks):
    return "<table {}>{}</table>".format(ks.get('attr', ""), "".join(map(str, e)))

dat = []
def printRow(tfdir):
    tf = os.path.basename(tfdir)
    #print >>sys.stderr, tf

    with open(os.path.join('../yeastfit/fit/', tf, 'gridHessian')) as f:
        hessiandat = array([[float(x) for x in l.split()] for l in f.readlines()[13:17]])
        hessiandat = hessiandat[argsort(hessiandat[:,0])]

    hessianTable = table(*([tr(th(lambdatxt), th(lognutxt), th(mutxt), th(logbetatxt), th(Tf0txt))] + 
                           [tr(*[td(printSci(x)) for x in d]) for d in hessiandat]), attr='class="params"')

    return tr(td(div(tf, attr='style="text-align: center; padding-bottom:40px;"') + hessianTable), td('<img height="250px" src="do3/{}.svg">'.format(tf)))

print """<!DOCTYPE html>
<html>
<head>
<style>
img{
}
td{
    border-width: 1px;
    border-style: solid;
}
table {
    border:none;
    border-collapse: collapse;
}

table td{
    border-left: none;
    border-right: none;
}

table.params{
    pargin: 10px;
}

table.params td{
    border: none;
    border-right: solid;
    border-width: 1px;
    padding: 1px 5px 1px 20px;
    text-align: right;
}

table.params th{
    border: none;
    border-right: solid;
    border-bottom: solid;
    border-width: 1px;
    text-align: right;
    padding: 1px 5px 1px 20px;
}


</style>
</head>
<body>
"""
header =  tr(th('(A) Hessian Eigenvectors'), 
             th('(B) Subsample Fits'),
            )

print table(*([header] + [printRow(fn) for fn in tfdirs]))

print """
</body>
</html>
"""
